//
// Xi Guo 2009
// used with TX.c for 13.56 MHz communication Demo
// 16 kHz baud rate
// A.7 = 14th bit
// A.6 = 13th bit
// A.5 = 12th bit
// A.4 = 11th bit
// A.3 = 10th bit
// A.2 = 9th bit
// A.1 = 8th bit
// A.0 = 7th bit
// PortB.3 = Vcc/2
// PortB.2 = 7th bit of counter for analog comparator interrupt
// PortD = led
       
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#define F_CPU 16000000UL  // 1 MHz
#include <util/delay.h>
#include <string.h>
#include <stdlib.h>
#include <avr/pgmspace.h>





#define begin {
#define end }

#define mode_highsync 1
#define mode_lowsync 2
#define mode_wait 3
#define mode_data 4

 
//the task subroutine
void task1(void);
void task2(void);

void initialize(void); //all the usual mcu stuff 

//timeout counter for task1         
volatile unsigned char led, inreg, count; 

// timer 1 capture variables for computing sq wave period	
volatile unsigned int T1capture, lastT1capture, period; 
volatile int time1, time2 ;
volatile unsigned char mode, bit, last;

// polling variables for computing sq wave period (low accuracy)
unsigned int periodPoll, T1poll, lastT1poll ;

//comparator output bit
char ACObit, lastACObit ;
         
//**********************************************************
//timer 0 overflow ISR
ISR (TIMER0_COMPA_vect) 
begin      

  if (mode == mode_highsync) {  
  	if (time1>0)	{
		--time1;
		if (PINA >= 0x40) {
			time2 = 10;
			mode = mode_lowsync;

		}

	}
	else {
		time1 = 10;
		PORTB = 0x01;

		_delay_us(0.2);
		PORTB = 0x00;
	}
  }
  else if (mode == mode_lowsync) {  
  	if (time2>0)	{
		--time2;
	}
	else {
		PORTB = 0x01;
		_delay_us(0.2);
		PORTB = 0x00;
		ACSR |= (1<<ACIE);
		inreg = 0;
		count = 0;
		mode = mode_wait;
		time2 = 20;
	}
  }
  else if (mode == mode_data) {  
	if (bit == 0x00) {
		last = PINA;
		bit = 0x01;
	}
	else {
		bit = 0x00;
		if (last < PINA)
			inreg = (inreg << 1) | 1;
		else if (last > PINA)
			inreg = (inreg << 1);
		
	}
	
	PORTB = 0x01;
	_delay_us(0.2);
	PORTB = 0x00;
  	if (++count > 16) {
  		led = inreg;
		mode = mode_highsync;
		time1 = 10;
	}

  }

  else if (mode == mode_wait) {  
  	if (time2>0)	{
		--time2;
	}
	else {
		ACSR &= ~(1<<ACIE);
		PORTB = 0x01;
		_delay_us(0.2);
		PORTB = 0x00;		
		time1 = 10;
		mode = mode_highsync;
	}
  }
end  

//**********************************************************
// timer 1 capture ISR
ISR (ANALOG_COMP_vect)
begin
	ACSR &= ~(1<<ACIE);

	_delay_us(53);

	PORTB = 0x01;
	_delay_us(0.2);
	PORTB = 0x00;
	bit = 0x01;
	mode = mode_data;
	TCNT0 = 10;

end
	

//**********************************************************       
//Entry point and task scheduler loop
int main(void)
begin  
  initialize();
  // main task scheduler loop 
  while(1)
  begin  
	PORTD = led;
  end
end  
  

 
//********************************************************** 
//Set it all up
void initialize(void)
begin
           
  led = 0x00;
  //set up timer 0 for 1 mSec timebase 
  TIMSK0= (1<<OCIE0A);	//turn on timer 0 cmp match ISR 
  OCR0A = 124;  		//set the compare re to 125 time ticks
  //set prescalar to divide by 8 
  TCCR0B= 2; //0b00000010;	
  // turn on clear-on-match
  TCCR0A= (1<<WGM01) ;
  
  //capture an edge on analog comparator pin B.3 
  // Set capture to positive edge, full counting rate
  // Set analog comp to connect to timer capture input 
  // and turn on the band gap reference on the positive input  
  ACSR = 3;
  // Comparator negative input is B.3
  DDRB = 0x03 ;
  

  DDRD = 0xff;
  DDRC = 0xff;
  DDRA = 0;  
  
  //init the task timer
  time1=10;
  mode = mode_highsync;
      
      
  //crank up the ISRs
  sei();

end  
//==================================================

   
