//
// Xi Guo 2009
// used with RX.c for 13.56 MHz communication Demo
// 16 kHz baud rate
// PortA = push button
// PortC.0 = enable on tri-state buffer
//


#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdio.h>
#include <util/delay.h>


#define begin {
#define end }

#define mode_highsync 1
#define mode_lowsync 2
#define mode_wait 3
#define mode_data 4

 
void initialize(void); //all the usual mcu stuff 
      
volatile unsigned char outreg, count; 
volatile int time1, time2 ;
volatile char mode, bit;


//**********************************************************
//timer 0 overflow ISR
ISR (TIMER0_COMPA_vect) 
begin      

  if (mode == mode_highsync) {  
  	if (time1>0)	{

		--time1;	
	}
	else {
		mode = mode_lowsync;
		PORTC = 0x01;
		time2 = 10;
	}
  }
  else if (mode == mode_lowsync) {  
  	if (time2>0)	{
		--time2;
	}
	else {
		PORTC = 0x00;
		count = 0;
		outreg = PINA;
		mode = mode_data;
		bit = 0x00;
	}
  }
  else if (mode == mode_data) {    

	if ((outreg & 0x80) == 0x00)
		PORTC = bit;
	else
		PORTC = (0x01 ^ bit);
	if (bit == 0x01) {
		outreg = outreg << 1;
		bit = 0x00;
	}
	else bit = 0x01;
	if (++count > 16) {
		mode = mode_wait;
		PORTC = 0x01;
		time1 = 100;		
	}	

  }
  else if (mode == mode_wait) {
   	if (time1>0)	{
		--time1;	
	}
	else {
		PORTC = 0x00;
		mode = mode_highsync;
		time1 = 12;
	}
  }
end  


//**********************************************************       
//Entry point and task scheduler loop
int main(void)
begin  
  initialize();
  
  // main task scheduler loop 
  while(1)
  begin

  end
end  
  

 
//********************************************************** 
//Set it all up
void initialize(void)
begin
           
  TIMSK0= (1<<OCIE0A);	//turn on timer 0 cmp match ISR 
  OCR0A = 124;  		//set the compare re to 125 time ticks
  //set prescalar to divide by 8 
  TCCR0B= 2; //0b00000010;	
  // turn on clear-on-match
  TCCR0A= (1<<WGM01) ;
 
  DDRB = 0 ;
  
  DDRC = 0xff;
  DDRA = 0x00;
  
  time1=10;
  mode = mode_highsync;
      
  //crank up the ISRs
  sei();

end  
//==================================================

   
